package org.example;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.stereotype.Service;
import redis.clients.jedis.DefaultJedisClientConfig;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.GeneralSecurityException;
import java.security.KeyStore;

@Service
public class RedisService
{
    //TODO: remove deprecated URL constructor
    //TODO: replace mapper towards concrete class model GameInfoResponse
    public JsonNode loadGameJson(String url) throws IOException
    {
        InputStream inputStream;

        Proxy proxy = new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(SteamConstants.PROXY_HOST, SteamConstants.PROXY_PORT));
        Authenticator authenticator = new Authenticator()
        {
            public PasswordAuthentication getPasswordAuthentication()
            {
                return (new PasswordAuthentication(SteamConstants.PROXY_USER, SteamConstants.PROXY_PASS.toCharArray()));
            }
        };
        Authenticator.setDefault(authenticator);
        URLConnection connection = new URL(url).openConnection(proxy);
        try
        {
            inputStream = connection.getInputStream();
        }
        catch (IOException exception)
        {
            int statusCode = ((HttpURLConnection)connection).getResponseCode();
            throw new IOException(String.valueOf(statusCode));
        }

        InputStreamReader reader = new InputStreamReader(inputStream);
        ObjectMapper mapper = new ObjectMapper();
        return mapper.readTree(reader);

    }

    public DefaultJedisClientConfig prepareRedisConfig() throws GeneralSecurityException, IOException
    {
        //Hack for building absolute path for user certificate
        String keystorePath = new File("src/main/resources/redis-keystore.p12").getAbsolutePath();

        //Hack for building absolute path for CA certificate
        String truststorePath = new File("src/main/resources/truststore.jks").getAbsolutePath();

        SSLSocketFactory sslFactory = createSslSocketFactory(truststorePath, keystorePath);

        return DefaultJedisClientConfig.builder().ssl(true).sslSocketFactory(sslFactory).user("default").password(SteamConstants.REDIS_PASS).build();
    }

    private static SSLSocketFactory createSslSocketFactory(String caCertPath, String userCertPath) throws IOException, GeneralSecurityException
    {

        KeyStore keyStore = KeyStore.getInstance("pkcs12");
        keyStore.load(Files.newInputStream(Paths.get(userCertPath)), SteamConstants.USER_CERT_PASS.toCharArray());

        KeyStore trustStore = KeyStore.getInstance("jks");
        trustStore.load(Files.newInputStream(Paths.get(caCertPath)), SteamConstants.CA_CERT_PASS.toCharArray());

        TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance("X509");
        trustManagerFactory.init(trustStore);

        KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance("PKIX");
        keyManagerFactory.init(keyStore, SteamConstants.USER_CERT_PASS.toCharArray());

        SSLContext sslContext = SSLContext.getInstance("TLS");
        sslContext.init(keyManagerFactory.getKeyManagers(), trustManagerFactory.getTrustManagers(), null);

        return sslContext.getSocketFactory();
    }
}
